@@ -3,35 +3,36 @@ from contextlib import contextmanager
 
 import psycopg2
 import psycopg2.extensions
-import six
 
-from dagster import Field, IntSource, StringSource, check, resource
+from dagster import Field, IntSource, StringSource
+from dagster import _check as check
+from dagster import resource
 
 
 class RedshiftError(Exception):
     pass
 
 
-class _BaseRedshiftResource(six.with_metaclass(abc.ABCMeta)):
+class _BaseRedshiftResource(abc.ABC):
     def __init__(self, context):  # pylint: disable=too-many-locals
         # Extract parameters from resource config
         self.conn_args = {
             k: context.resource_config.get(k)
             for k in (
-                'host',
-                'port',
-                'user',
-                'password',
-                'database',
-                'schema',
-                'connect_timeout',
-                'sslmode',
+                "host",
+                "port",
+                "user",
+                "password",
+                "database",
+                "schema",
+                "connect_timeout",
+                "sslmode",
             )
             if context.resource_config.get(k) is not None
         }
 
-        self.autocommit = context.resource_config.get('autocommit')
-        self.log = context.log_manager
+        self.autocommit = context.resource_config.get("autocommit")
+        self.log = context.log
 
     @abc.abstractmethod
     def execute_query(self, query, fetch_results=False, cursor_factory=None, error_callback=None):
@@ -46,7 +47,7 @@ class _BaseRedshiftResource(six.with_metaclass(abc.ABCMeta)):
 
 class RedshiftResource(_BaseRedshiftResource):
     def execute_query(self, query, fetch_results=False, cursor_factory=None, error_callback=None):
-        '''Synchronously execute a single query against Redshift. Will return a list of rows, where
+        """Synchronously execute a single query against Redshift. Will return a list of rows, where
         each row is a tuple of values, e.g. SELECT 1 will return [(1,)].
 
         Args:
@@ -65,26 +66,26 @@ class RedshiftResource(_BaseRedshiftResource):
         Returns:
             Optional[List[Tuple[Any, ...]]]: Results of the query, as a list of tuples, when
                 fetch_results is set. Otherwise return None.
-        '''
-        check.str_param(query, 'query')
-        check.bool_param(fetch_results, 'fetch_results')
-        check.opt_subclass_param(cursor_factory, 'cursor_factory', psycopg2.extensions.cursor)
-        check.opt_callable_param(error_callback, 'error_callback')
+        """
+        check.str_param(query, "query")
+        check.bool_param(fetch_results, "fetch_results")
+        check.opt_class_param(
+            cursor_factory, "cursor_factory", superclass=psycopg2.extensions.cursor
+        )
+        check.opt_callable_param(error_callback, "error_callback")
 
         with self._get_conn() as conn:
             with self._get_cursor(conn, cursor_factory=cursor_factory) as cursor:
                 try:
-                    six.ensure_str(query)
-
-                    self.log.info('Executing query \'{query}\''.format(query=query))
+                    self.log.info("Executing query '{query}'".format(query=query))
                     cursor.execute(query)
 
                     if fetch_results and cursor.rowcount > 0:
                         return cursor.fetchall()
                     else:
-                        self.log.info('Empty result from query')
+                        self.log.info("Empty result from query")
 
-                except Exception as e:  # pylint: disable=broad-except
+                except Exception as e:
                     # If autocommit is disabled or not set (it is disabled by default), Redshift
                     # will be in the middle of a transaction at exception time, and because of
                     # the failure the current transaction will not accept any further queries.
@@ -105,7 +106,7 @@ class RedshiftResource(_BaseRedshiftResource):
     def execute_queries(
         self, queries, fetch_results=False, cursor_factory=None, error_callback=None
     ):
-        '''Synchronously execute a list of queries against Redshift. Will return a list of list of
+        """Synchronously execute a list of queries against Redshift. Will return a list of list of
         rows, where each row is a tuple of values, e.g. ['SELECT 1', 'SELECT 1'] will return
         [[(1,)], [(1,)]].
 
@@ -125,29 +126,29 @@ class RedshiftResource(_BaseRedshiftResource):
         Returns:
             Optional[List[List[Tuple[Any, ...]]]]: Results of the query, as a list of list of
                 tuples, when fetch_results is set. Otherwise return None.
-        '''
-        check.list_param(queries, 'queries', of_type=str)
-        check.bool_param(fetch_results, 'fetch_results')
-        check.opt_subclass_param(cursor_factory, 'cursor_factory', psycopg2.extensions.cursor)
-        check.opt_callable_param(error_callback, 'error_callback')
+        """
+        check.list_param(queries, "queries", of_type=str)
+        check.bool_param(fetch_results, "fetch_results")
+        check.opt_class_param(
+            cursor_factory, "cursor_factory", superclass=psycopg2.extensions.cursor
+        )
+        check.opt_callable_param(error_callback, "error_callback")
 
         results = []
         with self._get_conn() as conn:
             with self._get_cursor(conn, cursor_factory=cursor_factory) as cursor:
                 for query in queries:
-                    six.ensure_str(query)
-
                     try:
-                        self.log.info('Executing query \'{query}\''.format(query=query))
+                        self.log.info("Executing query '{query}'".format(query=query))
                         cursor.execute(query)
 
                         if fetch_results and cursor.rowcount > 0:
                             results.append(cursor.fetchall())
                         else:
                             results.append([])
-                            self.log.info('Empty result from query')
+                            self.log.info("Empty result from query")
 
-                    except Exception as e:  # pylint: disable=broad-except
+                    except Exception as e:
                         # If autocommit is disabled or not set (it is disabled by default), Redshift
                         # will be in the middle of a transaction at exception time, and because of
                         # the failure the current transaction will not accept any further queries.
@@ -170,15 +171,19 @@ class RedshiftResource(_BaseRedshiftResource):
 
     @contextmanager
     def _get_conn(self):
+        conn = None
         try:
             conn = psycopg2.connect(**self.conn_args)
             yield conn
         finally:
-            conn.close()
+            if conn:
+                conn.close()
 
     @contextmanager
     def _get_cursor(self, conn, cursor_factory=None):
-        check.opt_subclass_param(cursor_factory, 'cursor_factory', psycopg2.extensions.cursor)
+        check.opt_class_param(
+            cursor_factory, "cursor_factory", superclass=psycopg2.extensions.cursor
+        )
 
         # Could be none, in which case we should respect the connection default. Otherwise
         # explicitly set to true/false.
@@ -200,7 +205,7 @@ class FakeRedshiftResource(_BaseRedshiftResource):
     QUERY_RESULT = [(1,)]
 
     def execute_query(self, query, fetch_results=False, cursor_factory=None, error_callback=None):
-        '''Fake for execute_query; returns [self.QUERY_RESULT]
+        """Fake for execute_query; returns [self.QUERY_RESULT]
 
         Args:
             query (str): The query to execute.
@@ -218,20 +223,22 @@ class FakeRedshiftResource(_BaseRedshiftResource):
         Returns:
             Optional[List[Tuple[Any, ...]]]: Results of the query, as a list of tuples, when
                 fetch_results is set. Otherwise return None.
-        '''
-        check.str_param(query, 'query')
-        check.bool_param(fetch_results, 'fetch_results')
-        check.opt_subclass_param(cursor_factory, 'cursor_factory', psycopg2.extensions.cursor)
-        check.opt_callable_param(error_callback, 'error_callback')
+        """
+        check.str_param(query, "query")
+        check.bool_param(fetch_results, "fetch_results")
+        check.opt_class_param(
+            cursor_factory, "cursor_factory", superclass=psycopg2.extensions.cursor
+        )
+        check.opt_callable_param(error_callback, "error_callback")
 
-        self.log.info('Executing query \'{query}\''.format(query=query))
+        self.log.info("Executing query '{query}'".format(query=query))
         if fetch_results:
             return self.QUERY_RESULT
 
     def execute_queries(
         self, queries, fetch_results=False, cursor_factory=None, error_callback=None
     ):
-        '''Fake for execute_queries; returns [self.QUERY_RESULT] * 3
+        """Fake for execute_queries; returns [self.QUERY_RESULT] * 3
 
         Args:
             queries (List[str]): The queries to execute.
@@ -249,116 +256,113 @@ class FakeRedshiftResource(_BaseRedshiftResource):
         Returns:
             Optional[List[List[Tuple[Any, ...]]]]: Results of the query, as a list of list of
                 tuples, when fetch_results is set. Otherwise return None.
-        '''
-        check.list_param(queries, 'queries', of_type=str)
-        check.bool_param(fetch_results, 'fetch_results')
-        check.opt_subclass_param(cursor_factory, 'cursor_factory', psycopg2.extensions.cursor)
-        check.opt_callable_param(error_callback, 'error_callback')
+        """
+        check.list_param(queries, "queries", of_type=str)
+        check.bool_param(fetch_results, "fetch_results")
+        check.opt_class_param(
+            cursor_factory, "cursor_factory", superclass=psycopg2.extensions.cursor
+        )
+        check.opt_callable_param(error_callback, "error_callback")
 
         for query in queries:
-            self.log.info('Executing query \'{query}\''.format(query=query))
+            self.log.info("Executing query '{query}'".format(query=query))
         if fetch_results:
             return [self.QUERY_RESULT] * 3
 
 
 def define_redshift_config():
-    '''Redshift configuration. See the Redshift documentation for reference:
+    """Redshift configuration. See the Redshift documentation for reference:
 
     https://docs.aws.amazon.com/redshift/latest/mgmt/connecting-to-cluster.html
-    '''
+    """
 
     return {
-        'host': Field(StringSource, description='Redshift host', is_required=True),
-        'port': Field(
-            IntSource, description='Redshift port', is_required=False, default_value=5439
+        "host": Field(StringSource, description="Redshift host", is_required=True),
+        "port": Field(
+            IntSource, description="Redshift port", is_required=False, default_value=5439
         ),
-        'user': Field(
-            StringSource, description='Username for Redshift connection', is_required=False,
-        ),
-        'password': Field(
-            StringSource, description='Password for Redshift connection', is_required=False,
-        ),
-        'database': Field(
+        "user": Field(
             StringSource,
-            description='Name of the default database to use. After login, you can use USE DATABASE'
-            ' to change the database.',
+            description="Username for Redshift connection",
             is_required=False,
         ),
-        'schema': Field(
+        "password": Field(
             StringSource,
-            description='Name of the default schema to use. After login, you can use USE SCHEMA to '
-            'change the schema.',
+            description="Password for Redshift connection",
             is_required=False,
         ),
-        'autocommit': Field(
+        "database": Field(
+            StringSource,
+            description="Name of the default database to use. After login, you can use USE DATABASE"
+            " to change the database.",
+            is_required=False,
+        ),
+        "schema": Field(
+            StringSource,
+            description="Name of the default schema to use. After login, you can use USE SCHEMA to "
+            "change the schema.",
+            is_required=False,
+        ),
+        "autocommit": Field(
             bool,
-            description='None by default, which honors the Redshift parameter AUTOCOMMIT. Set to '
-            'True or False to enable or disable autocommit mode in the session, respectively.',
+            description="None by default, which honors the Redshift parameter AUTOCOMMIT. Set to "
+            "True or False to enable or disable autocommit mode in the session, respectively.",
             is_required=False,
         ),
-        'connect_timeout': Field(
+        "connect_timeout": Field(
             int,
-            description='Connection timeout in seconds. 5 seconds by default',
+            description="Connection timeout in seconds. 5 seconds by default",
             is_required=False,
             default_value=5,
         ),
-        'sslmode': Field(
+        "sslmode": Field(
             str,
-            description='SSL mode to use. See the Redshift documentation for more information on '
-            'usage: https://docs.aws.amazon.com/redshift/latest/mgmt/connecting-ssl-support.html',
+            description="SSL mode to use. See the Redshift documentation for more information on "
+            "usage: https://docs.aws.amazon.com/redshift/latest/mgmt/connecting-ssl-support.html",
             is_required=False,
-            default_value='require',
+            default_value="require",
         ),
     }
 
 
 @resource(
-    config=define_redshift_config(),
-    description='Resource for connecting to the Redshift data warehouse',
+    config_schema=define_redshift_config(),
+    description="Resource for connecting to the Redshift data warehouse",
 )
 def redshift_resource(context):
-    '''This resource enables connecting to a Redshift cluster and issuing queries against that
+    """This resource enables connecting to a Redshift cluster and issuing queries against that
     cluster.
 
     Example:
 
         .. code-block:: python
 
-            from dagster import ModeDefinition, execute_solid, solid
+            from dagster import build_op_context, op
             from dagster_aws.redshift import redshift_resource
 
-            @solid(required_resource_keys={'redshift'})
-            def example_redshift_solid(context):
+            @op(required_resource_keys={'redshift'})
+            def example_redshift_op(context):
                 return context.resources.redshift.execute_query('SELECT 1', fetch_results=True)
 
-            result = execute_solid(
-                example_redshift_solid,
-                environment_dict={
-                    'resources': {
-                        'redshift': {
-                            'config': {
-                                'host': 'my-redshift-cluster.us-east-1.redshift.amazonaws.com',
-                                'port': 5439,
-                                'user': 'dagster',
-                                'password': 'dagster',
-                                'database': 'dev',
-                            }
-                        }
-                    }
-                },
-                mode_def=ModeDefinition(resource_defs={'redshift': redshift_resource}),
-            )
-            assert result.output_value() == [(1,)]
+            redshift_configured = redshift_resource.configured({
+                'host': 'my-redshift-cluster.us-east-1.redshift.amazonaws.com',
+                'port': 5439,
+                'user': 'dagster',
+                'password': 'dagster',
+                'database': 'dev',
+            })
+            context = build_op_context(resources={'redshift': redshift_configured})
+            assert example_redshift_op(context) == [(1,)]
 
-    '''
+    """
     return RedshiftResource(context)
 
 
 @resource(
-    config=define_redshift_config(),
-    description='Fake resource for connecting to the Redshift data warehouse. Usage is identical '
-    'to the real redshift_resource. Will always return [(1,)] for the single query case and '
-    '[[(1,)], [(1,)], [(1,)]] for the multi query case.',
+    config_schema=define_redshift_config(),
+    description="Fake resource for connecting to the Redshift data warehouse. Usage is identical "
+    "to the real redshift_resource. Will always return [(1,)] for the single query case and "
+    "[[(1,)], [(1,)], [(1,)]] for the multi query case.",
 )
 def fake_redshift_resource(context):
     return FakeRedshiftResource(context)
